#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <unistd.h>

#include <libpv.h>

static char *prompt="netXio";
static int shutdown = 0;

void signalhandler (int sig)
{
	if (sig == SIGPIPE) return;
	if (sig == SIGHUP) {
		fprintf(stderr, "%s SIGHUP received\n", prompt);
	}
	if (sig == SIGTERM || sig == SIGINT) {
		fprintf(stderr, "%s received SIGTERM or SIGINT, shutting down\n", prompt);
		shutdown = 1;
	}
}

int main(int argc, char *argv[])
{
	struct sigaction	action;
	pv_engine_t 		pv_engine;
	pv_engine_attr_t	attr;
	int			err;
	int			i;

	/* hardware */
	uint32_t		*pv_leds_set;
	uint32_t		*pv_leds_clear;
	uint32_t		*pv_switches;

	/* logical stuff */
	uint8_t			*pv_level;
	uint8_t			*pv_fill;

	err = pv_engine_attr_init_from_cmdline(&attr, argc, argv);
	if (err)
		exit (EXIT_FAILURE);

	err = pv_init(&pv_engine, &attr);
	if (err) 
		exit (EXIT_FAILURE);

	printf("%s: engine created\n", prompt);

	/* setup signal handler */
	action.sa_handler = signalhandler;
	sigemptyset (&action.sa_mask);
	action.sa_flags = 0;
	sigaction (SIGTERM, &action, 0);	/* catches kill from the shell   */
	sigaction (SIGINT,  &action, 0);	/* catches Ctrl-C from the shell */
	sigaction (SIGPIPE, &action, 0);	/* catches Broken Pipe exception */
	sigaction (SIGHUP,  &action, 0);	/* catches HUP                   */

	/* register PVs */
	pv_leds_set = pv_register_uint32(&pv_engine, "leds_set");
	pv_leds_clear = pv_register_uint32(&pv_engine, "leds_clear");
	pv_switches = pv_register_uint32(&pv_engine, "switches");
	
	pv_level = pv_register_uint8(&pv_engine, "level");
	pv_fill = pv_register_uint8(&pv_engine, "fill");

	/* check if we could register everything */
	if ((pv_leds_set == NULL) || (pv_leds_clear == NULL) || (pv_switches == NULL) ||
	    (pv_level == NULL) || (pv_fill == NULL)) {
		fprintf(stderr, "could not register variables\n");
		pv_destroy(&pv_engine);
		exit(EXIT_FAILURE);
	}		

	/* use standard ASCII device functions */
        pv_set_fn_write(&pv_engine, pv_leds_set, pv_device_write_int_file_ascii);
        pv_set_fn_write(&pv_engine, pv_leds_clear, pv_device_write_int_file_ascii);
	pv_set_fn_read(&pv_engine, pv_switches, pv_device_read_int_file_ascii);

	while (!shutdown) {

		pv_lock(&pv_engine);

		/* update inputs from hardware */
		pv_from_device(&pv_engine, pv_switches);

		/* FIXME: update some internal pvs here */

//		printf("pv_fill=%i pv_level=%2i\n", *pv_fill, *pv_level);

		if (*pv_level > 15)
			*pv_level=15;
//		if (*pv_level < 0)
//			*pv_level=0;
		
		/* sed LEDs from pv_fill */
		*pv_leds_set = (1<<(*pv_level));
//		printf("pv_leds_set   = 0x%08x\n", (*pv_leds_set));
		
		if (*pv_fill) {
			for (i=0; i<(*pv_level); i++) {
				(*pv_leds_set) |= (1<<i);
			}
		}

		(*pv_leds_clear) = (-(*pv_leds_set + 1)) & 0xffff;

		pv_to_device(&pv_engine, pv_leds_set);
		pv_to_device(&pv_engine, pv_leds_clear);

		pv_unlock(&pv_engine);
		
			usleep(10000);
		
	}
	
	pv_destroy(&pv_engine);
	
	return EXIT_SUCCESS;
}

