/*
 * file: pv_attr.c
 * This file is part of the libpv library.
 *
 * Copyright 2005 Pengutronix
 *
 * Please send bug reports to bugs@pengutronix.de
 *
 * See COPYING for the status of this software.
 */

#include <libpv_config.h>

#include <errno.h>
#include <getopt.h>
#include <libgen.h>
#include <string.h>

#include <pv_types.h>
#include <libpv.h>
#include <jv_server.h>

/**
 * pv_engine_attr_init
 * @attr: Pointer to structure to clear
 *
 * Clears all attributes
 *
 * Return value: EINVAL in case of error, 0 elsewhere
 **/
int pv_engine_attr_init(pv_engine_attr_t	*attr)
{
	if (!attr)
		return -EINVAL;

	memset(attr, 0, sizeof(struct pv_engine_attr));

	attr->store_type = PV_STORE_STANDARD;

	return 0;
}

/**
 * show_help
 * @name: Pointer to the program name
 *
 * Shows small help how to define attributes
 * on command line
 *
 * Return value: nothing
 **/
static void show_help(char *name)
{
	fprintf(stderr,
		"\nusage:   "
		"%s [-v] -d <file> -s <file> [-j <port>]\n\n"
		"  -v, --verbose             Verbose mode of operation\n"
		"  -d, --description <file>  Description XML File: specifies layout of the \n"
		"                            process variable store file\n"
		"  -s, --store <file>        Store file: this file holds the process variables\n"
		"  -8, --8bit                use 8 bit backing store\n"
		"  -j, --jvisu [<port>]      Start JVisu Socket Server, default %d\n"
		"\n",
		name, JVISU_PORT_DEFAULT);
}

/**
 * pv_engine_attr_init_from_cmdline
 * @attr: Structure to store attributes
 * @argc: Argument's count from command line
 * @argv: Arguments from command line
 *
 * Parses given command line options and parameters
 * to extract attributes for this run
 *
 * Return value: EINVAL in case of parameter error, 0 elsewhere
 **/
int pv_engine_attr_init_from_cmdline(pv_engine_attr_t	*attr,
				     int		argc,
				     char		*argv[])
{
	struct option	long_options[] = {
		{ "help",       no_argument,       0, 'h' },
		{ "verbose",    no_argument,       0, 'v' },
		{ "desc", 	required_argument, 0, 'd' },
		{ "store",      required_argument, 0, 's' },
		{ "8bit",       no_argument,       0, '8' },
		{ 0,            0,                 0,  0  },
	};
	char		*optdesc = NULL;
	char		*optstore = NULL;
	char		*optjvport = NULL;
	int		longindex;
	int		opt;

	if (!attr)
		return -EINVAL;

	memset(attr, 0, sizeof(struct pv_engine_attr));

	/* reset getopt */
	optind = 0;

	/* parse command line switches */
	while ((opt = getopt_long(argc, argv, "hvd:s:8j::", long_options, &longindex)) != -1) {
		switch (opt) {
		case 'h':
			show_help(basename(argv[0]));
			return -1;
		case 'v':
			_libpv_debuglevel++;	/* FIXME: move this into pv_engine struct (mkl) */
			break;
		case 'd':
			optdesc = optarg;
			break;
		case 's':
			optstore = optarg;
			break;
		case 'j':
			optjvport = optarg;
			break;
		case '8':
			attr->store_type = PV_STORE_8BIT;
			break;
                default:
                        show_help(argv[0]);
			return -EINVAL;
                }
	}

	if (!optdesc || !optstore) {
		show_help(argv[0]);
		return -EINVAL;
	}

	if (optjvport)
		attr->jvisu_port = strtoul(optjvport, NULL, 0);
	else
		attr->jvisu_port = JVISU_PORT_DEFAULT;

	strncpy(attr->store_filename, optstore, NAME_MAX);
	attr->store_filename[NAME_MAX] = 0;

	strncpy(attr->desc_filename, optdesc, NAME_MAX);
	attr->desc_filename[NAME_MAX] = 0;

	return 0;
}

/* end of file pv_attr.c */
