/*
 * file: pv_device.c
 * This file is part of the libpv library.
 *
 * Copyright 2005 Pengutronix
 *
 * Please send bug reports to bugs@pengutronix.de
 *
 * See COPYING for the status of this software.
 */

#define _XOPEN_SOURCE 500

#include <libpv_config.h>

#include <errno.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <unistd.h>

#include <libpv.h>

#include <pv_default.h>
#include <pv_types.h>
#include <pv_get.h>
#include <pv_set.h>
#include <pv_bs.h>

/*
 * FIXME:
 *
 * - different return values in to_device/from_device
 * - make a macro from pv_to_device / pv_from_device
 * - API documentation
 * - all these functions are nearly identical in how they find out about
 *   their pv -> unify.
 * - document locking policy: the from/to_device functions must be
 *   called with a locked process image (right?)
 */

/**
 * pv_to_device - call the connected write function to update external resources
 * @engine_anon: Current process image to work with
 * @pv_var: process variable to work for
 *
 * Description: pv_to_device() updates an external resource on the base of
 * a process variable at offset @pv_var. The connected write function is called
 * and does the device specific things. Due to the fact this call modifies
 * process variables it should only called within a locked process image.
 *
 * Return value: 0 on success, ENOSYS if not write function is connected
 * and ENXIO if there are no process variable found for @pv_var
 **/
int pv_to_device(pv_engine_t *engine_anon, void *pv_var)
{
	struct pv_engine	*engine = (struct pv_engine*)engine_anon;
	struct pv		*pv;
	off_t			pv_var_offset = abs_addr_to_offset(engine,pv_var);
	int			finished = 0;

	for (goto_head(&engine->pv_list); !finished; goto_next(&engine->pv_list)) {

		if (is_last(engine->pv_list))
			finished=1;

		pv = (struct pv *)list_get_element_data(engine->pv_list);
		if (pv->offset == pv_var_offset) {

			if (!pv->device_write)
				return -ENOSYS;	/* there is no write function connected */

			pv->device_write(engine_anon,pv, 0);

			return 0;	/* ok, updated */
		}
	}

	return -ENXIO;	/* pv_var does not point to any known process variable */
}

/**
 * pv_from_device - call the connected read function to update a process variable
 * @engine_anon: Current process image to work with
 * @pv_var: process variable to work for
 *
 * Description: pv_from_device() updates the process variable at offset @pv_var
 * from an external source by calling process variable's connected read function.
 * The read function does the device specific things here to generate a new value
 * for this process variable. Due to the fact this call modifies
 * process variables it should only called within a locked process image.
 *
 * Return value: 0 on success, ENOSYS if not write function is connected
 * and ENXIO if there are no process variable found for @pv_var
 **/
int pv_from_device(pv_engine_t *engine_anon, void *pv_var)
{
	struct pv_engine	*engine = (struct pv_engine*)engine_anon;
	struct pv		*pv;
	off_t			pv_var_offset = abs_addr_to_offset(engine,pv_var);
	int			finished = 0;

	for (goto_head(&engine->pv_list); !finished; goto_next(&engine->pv_list)) {

		if (is_last(engine->pv_list))
			finished=1;

		pv = (struct pv *)list_get_element_data(engine->pv_list);
		if (pv->offset == pv_var_offset) {

			if (!pv->device_read)
				return -ENOSYS;	/* there is no read function connected */

			pv->device_read(engine_anon,pv, 0);

			return 0;	/* ok, updated */
		}
	}

	return -ENXIO;	/* pv_var does not point to any known process variable */
}

/**
 * pv_set_fn_read - connect a read function to process variable
 * @engine_anon: Current process image to work with
 * @pv_var: Process variable to connect (absolute address)
 * @device_read: Pointer to a read function
 *
 * Description: pv_set_fn_read() connects a read function to a process variable.
 * This connected function gets called whenever pv_from_device() is called with
 * this process variable. Use this feature when this process variable depends on
 * resources outside the process image like hardware registers for example.
 *
 * Return values: 0 on success, ENXIO if there are no process
 * variable found for @pv_var
 **/
int pv_set_fn_read(pv_engine_t *engine_anon, void *pv_var, pv_update_callback_fn_t device_read)
{
	struct pv_engine	*engine = (struct pv_engine*)engine_anon;
	struct pv		*pv;
	off_t			pv_var_offset = abs_addr_to_offset(engine,pv_var);
	int			finished = 0;

	for (goto_head(&engine->pv_list); !finished; goto_next(&engine->pv_list)) {
	/*
	 * Find the process variable element at the given
	 * offset and register the read function
	 */
		if (is_last(engine->pv_list))
			finished=1;

		pv = (struct pv *)list_get_element_data(engine->pv_list);
		if (pv->offset == pv_var_offset) {
		/* FIXME: Test if there is already a read function registered! */
			pv->device_read = device_read;
			return 0;
		}
	}

	return -ENXIO;	/* pv_var does not point to any known process variable */
}

/**
 * pv_set_fn_write - connect a write function to a process variable
 * @engine_anon: Current process image to work with
 * @pv_var: Process variable to connect (absolute address)
 * @device_write: Pointer to a write function
 *
 * Description: pv_set_fn_write() connects a write function to a process variable.
 * This connected function gets called whenever pv_to_device() is called with
 * this process variable. Use this feature to ensures that a changed process
 * variable value can also change dependend resources outside the process image like
 * a hardware register or other process variables.
 *
 * Return value: 0 on success, ENXIO if there are no process variable
 * found for @pv_var
 **/
int pv_set_fn_write(pv_engine_t *engine_anon, void *pv_var, pv_update_callback_fn_t device_write)
{
	struct pv_engine	*engine = (struct pv_engine*)engine_anon;
	struct pv		*pv;
	off_t			pv_var_offset = abs_addr_to_offset(engine,pv_var);
	int			finished = 0;

	for (goto_head(&engine->pv_list); !finished; goto_next(&engine->pv_list)) {
	/*
	 * Find the process variable element at the given
	 * offset and register the write function
	 */
		if (is_last(engine->pv_list))
			finished=1;

		pv = (struct pv *)list_get_element_data(engine->pv_list);

		if (pv->offset == pv_var_offset) {
		/* FIXME: Test if there is already a write function registered! */
			pv->device_write = device_write;
			return 0;
		}
	}

	return -ENXIO; /* pv_var does not point to any known process variable */
}

/*
 * FIXME: This could be an example how to do it, but it is not a good idea
 * to let the functions below inside the libpv. They are too device specific!
 */

/**
 * pv_device_read_int_file_ascii - support function to update process variables
 * @engine_anon:  Current process image to work with
 * @pv_void: Process variable to update
 * @data: not used
 *
 * Description: You can use this support function in a pv_set_fn_read() call
 * to update the process variable in question with a value generated from a
 * textual string. Currently it only works with signed integer values!
 * The string is readed from the device defined in the XML description file.
 * Due to the fact this call modifies process variables it should only
 * called within a locked process image.
 * FIXME: make this also do float and string stuff and support unsigned integer
 *
 * Return value: 0 on success, ENODEV if no device is defined for this process
 * variable, -1 if process variable's type isn't supported and -errno if something
 * went wrong while reading from the device.
 **/
int pv_device_read_int_file_ascii(pv_engine_t *engine_anon, struct pv *pv, unsigned long data)
{
	struct pv_engine *engine = (struct pv_engine*)engine_anon;
	char buf[255];
	long ltmp;
	int ret;

	if (pv->device_file == -1)
		return -ENODEV;

	ret = pread(pv->device_file, buf, sizeof(buf), 0);
	if (ret < 0)
		return -errno;

	/* FIXME: quick hack... please fix long vs. ulong issues */
	ltmp = strtol(buf, NULL, 0);

	switch (pv->type) {
	case PV_TYPE_UINT8:
		pv_set_uint8(engine, pv, (uint8_t)(ltmp));
		break;;
	case PV_TYPE_UINT16:
		pv_set_uint16(engine, pv, (uint16_t)(ltmp));
		break;;
	case PV_TYPE_UINT32:
		pv_set_uint32(engine, pv, (uint32_t)(ltmp));
		break;;
	case PV_TYPE_INT8:
		pv_set_int8(engine, pv, (int8_t)(ltmp));
		break;;
	case PV_TYPE_INT16:
		pv_set_int16(engine, pv, (int16_t)(ltmp));
		break;;
	case PV_TYPE_INT32:
		pv_set_int32(engine, pv, (int32_t)(ltmp));
		break;;
	default:
		printf("unrecognized type\n");
		return -1;
	}

	return 0;
}

/**
 * pv_device_write_int_file_ascii - support function to update a devices based on a process variables
 * @engine_anon:  Current process image to work with
 * @pv_void: Process variable to update the device
 * @data: not used
 *
 * Description: You can use this support function in a pv_set_fn_write() call
 * to feed the value of process variable @pv_void into a device.
 * Currently it only works with signed integer values! The value is converted into a string
 * and its feeded into the device defined in the XML description file.
 * It is maybe a good idea to call this function within a locked process image.
 * FIXME: make this also do float and string stuff and support unsigned integer
 *
 * Return value: 0 on success, ENODEV if no device is defined for this process
 * variable, -1 if process variable's type isn't supported and -errno if something
 * went wrong while reading from the device.
 **/
int pv_device_write_int_file_ascii(pv_engine_t *engine_anon, struct pv *pv, unsigned long data)
{
	struct pv_engine *engine = (struct pv_engine*)engine_anon;
	char buf[255];
	long ltmp;
	int ret;

	if (pv->device_file == -1)
		return -ENODEV;

	/* FIXME: quick hack... please fix long vs. ulong issues */

	switch (pv->type) {
	case PV_TYPE_UINT8:
		ltmp = pv_get_uint8(engine, pv);
		break;;
	case PV_TYPE_UINT16:
		ltmp = pv_get_uint16(engine, pv);
		break;;
	case PV_TYPE_UINT32:
		ltmp = pv_get_uint32(engine, pv);
		break;;
	case PV_TYPE_INT8:
		ltmp = pv_get_int8(engine, pv);
		break;;
	case PV_TYPE_INT16:
		ltmp = pv_get_int16(engine, pv);
		break;;
	case PV_TYPE_INT32:
		ltmp = pv_get_int32(engine, pv);
		break;;
	default:
		printf("unrecognized type\n");
		return -1;
	}

	snprintf(buf, sizeof(buf), "%li\n", ltmp);

	ret = pwrite(pv->device_file, buf, sizeof(buf), 0);

	return ret > 0 ? 0 : -errno;
}

/* end of file pv_device.c */
